var Service = require('Service');
var util = require('util');
var path = require('path');
var fs = require('fs');
var utils = require('utils');
var TBSCompiler = require('TBSCompiler');

/**
 * 创建数据添加服务
 */
var CreateDataAddTBS = function() { 
  Service.call(this);
};

util.inherits(CreateDataAddTBS, Service);

module.exports = CreateDataAddTBS;

/**
 * 处理服务请求
 * @param  {Object} req 服务请求对象
 * @param  {Object} res 服务响应对象
 */
CreateDataAddTBS.prototype.process =  function(req, res) {
  if(!req.path) {
    this.onLogicError(1, '请提供服务路径！');
    return;
  }
  if(!req.type) {
    this.onLogicError(2, '请提供服务类型！');
    return;
  }
  if(!req.model) {
    this.onLogicError(3, '请提供数据模型名称！');
    return;
  }
  if(!req.fields) {
    this.onLogicError(4, '请提供字段信息！');
    return;
  }
  var filePath = utils.getPath(req.path);
  if(fs.existsSync(filePath)) {
    this.onLogicError(5, '文件已存在！');
    return;
  }
  var reqArgs = [];
  for(var i=0;i<req.fields.length;i++) {
    let field = req.fields[i];
    let reqArg = {
      name: field.name,
      type: "string"
    };
    if(field.notnull) reqArg.notNull = true;
    reqArgs.push(reqArg);
  }
  let tbsName = req.path.substr(req.path.lastIndexOf("/")+1);
  tbsName = tbsName.substr(0, tbsName.lastIndexOf("."));
  tbsName = tbsName.substr(0, 1).toUpperCase()+tbsName.substr(1);
  var tbs = {
    name: tbsName,
    comment: "添加"+req.model,
    reqArgs: reqArgs,
    retArgs: [{name:"id",type:"int"}],
    methods: [{
      name: "process",
      comment: "处理服务请求",
      args: [{
        name: "req",
        type: "object",
        comment: "服务请求对象"
      },{
        name: "res",
        type: "object",
        comment: "服务响应对象"
      }],
      statements: [{ 
        category: "db",
        type: "insert",
        model: {
          name: req.model,
          path: "/app/"+req.proj+"/model/"+req.model+".tdm"
        },
        fields: req.fields,
        cbArg: "ret", 
        statements: [{
          category: "common",
          type: "assign",
          varName: "res.id",
          operator: "=",
          value: "ret.insertId"
        },{
          category: "common",
          type: "end"
        }]
      }]
    }]
  };

  if(req.type=="tbs") {
    fs.writeFileSync(filePath, JSON.stringify(tbs, null, "\t"));
    this.end(res);
  } else {
    var self = this;
    var compiler = new TBSCompiler();
    compiler.process({tbs: tbs}, {code: 0}, function(res2) {
      if(res2.code!=0) {
        self.onLogicError(6, "TBS文件编译失败："+res2.message);
        return;
      }
      fs.writeFileSync(filePath, res2.jsCode);
      self.end(res);
    });
  }
};