var Service = require('Service');
var util = require('util');
var path = require('path');
var fs = require('fs');
var utils = require('utils');
var TBSCompiler = require('TBSCompiler');

/**
 * 创建数据查询服务
 */
var CreateDataQueryTBS = function() { 
  Service.call(this);
  this.sqlInject = {
    check: false   
  };
};

util.inherits(CreateDataQueryTBS, Service);

module.exports = CreateDataQueryTBS;

/**
 * 处理服务请求
 * @param  {Object} req 服务请求对象
 * @param  {Object} res 服务响应对象
 */
CreateDataQueryTBS.prototype.process =  function(req, res) {
  if(!req.path) {
    this.onLogicError(1, '请提供服务路径！');
    return;
  }
  if(!req.type) {
    this.onLogicError(2, '请提供服务类型！');
    return;
  }
  if(!req.model) {
    this.onLogicError(3, '请提供数据对象名称！');
    return;
  }
  if(!req.fields) {
    this.onLogicError(4, '请提供要查询的字段！');
    return;
  }
  var filePath = utils.getPath(req.path);
  if(fs.existsSync(filePath)) {
    this.onLogicError(5, '文件已存在！');
    return;
  }
  if(!req.retArg) req.retArg = "data";
  var dbQuery = { 
    category: "db",
    type: "query",
    model: {
      name: req.model,
      path: "/app/"+req.proj+"/model/"+req.model+".tdm"
    },
    fields: req.fields,
    cbArg: "rows", 
    statements: [{
      category: "common",
      type: "assign",
      varName: "res."+req.retArg,
      operator: "=",
      value: "rows"
    },{
      category: "common",
      type: "end"
    }]
  };
  var retArg = {name:req.retArg,type:"array"};
  if(req.orders) dbQuery.orders = req.orders;
  if(req.groups) dbQuery.groups = req.groups;
  if(req.where) dbQuery.where = req.where;
  if(req.pageSize) dbQuery.pageSize = parseInt(req.pageSize);
  if(req.count) dbQuery.count = parseInt(req.count);
  if(req.distinct) dbQuery.distinct = true;
  if(req.enableDataPage) {
    dbQuery.enableDataPage = req.enableDataPage;
    retArg = {name:req.retArg,type:"对象"};
  }
  let tbsName = req.path.substr(req.path.lastIndexOf("/")+1);
  tbsName = tbsName.substr(0, tbsName.lastIndexOf("."));
  tbsName = tbsName.substr(0, 1).toUpperCase()+tbsName.substr(1);
  var tbs = {
    name: tbsName,
    comment: "查询"+req.model,
    reqArgs: [],
    retArgs: [retArg],
    methods: [{
      name: "process",
      comment: "处理服务请求",
      args: [{
        name: "req",
        type: "object",
        comment: "服务请求对象"
      },{
        name: "res",
        type: "object",
        comment: "服务响应对象"
      }],
      statements: [dbQuery]
    }]
  };
  if(req.reqArgs) tbs.reqArgs = req.reqArgs;

  if(req.type=="tbs") {
    fs.writeFileSync(filePath, JSON.stringify(tbs, null, "\t"));
    this.end(res);
  } else {
    var self = this;
    var compiler = new TBSCompiler();
    compiler.process({tbs: tbs}, {code: 0}, function(res2) {
      if(res2.code!=0) {
        self.onLogicError(6, "TBS文件编译失败："+res2.message);
        return;
      }
      fs.writeFileSync(filePath, res2.jsCode);
      self.end(res);
    });
  }
};