var util = require('util');
var fs = require('fs');
var xlsx = require('node-xlsx');
var Service = require('Service');
var utils = require('utils');
var Dao = require('Dao');

/**
 * 导入Excel数据
 */
var ImportExcel = function () {
  Service.call(this);
};

util.inherits(ImportExcel, Service);

module.exports = ImportExcel;

/**
 * 处理服务请求
 * @param  {Object} req 服务请求对象
 * @param  {Object} res 服务响应对象
 */
ImportExcel.prototype.process = function (req, res, cb) {
  if (req.options) {
    if (!req.options.fileCode) {
      this.onLogicError(1, '请提供文件代码！');
      return;
    }
    this.parse(req, res, req.options.fileCode, cb);
  } else if (req.fileCode) {
    this.parse(req, res, req.fileCode, cb);
  } else {
    this.onLogicError(1, '请提供导入配置信息！');
  }
};

ImportExcel.prototype.parse = function (req, res, fileCode, cb) {
  var filePath = server.attachSavePath + '/tmp/upload_' + fileCode;
  if (!fs.existsSync(filePath)) {
    this.onLogicError(2, '文件不存在，请提供正确的文件代码！');
    return;
  }
  logger.log('正在解析Excel文件：' + filePath);
  var obj;
  try {
    obj = xlsx.parse(filePath);
  } catch (err) {
    this.onLogicError(3, '解析Excel文件失败！');
    return;
  }
  //删除临时文件
  /*try {
    fs.unlinkSync(filePath);
  } catch(err) {}*/
  //console.log(obj);
  if (obj.length == 0) {
    this.onLogicError(4, '未解析到有效的单元格！');
    return;
  }

  if (req.options) {
    //let sheetIndex = 0;
    //if(req.options.sheetIndex) sheetIndex = req.options.sheetIndex;
    this.import(req, res, obj[0].data, cb);
  } else if (req.fileCode) {
    res.data = obj[0].data;
    if (cb) {
      cb(req, res);
    } else {
      this.end(res);
    }
  } else {
    this.end(res);
  }
};

ImportExcel.prototype.import = function (req, res, data, cb) {
  req.options.titleRowIndex = parseInt(req.options.titleRowIndex);
  if (data.length < req.options.titleRowIndex) {
    this.onLogicError(5, '未解析到标题行！');
    return;
  }
  if (data.length == req.options.titleRowIndex) {
    this.onLogicError(6, '未解析到数据！');
    return;
  }
  logger.log("共解析到" + (data.length - req.options.titleRowIndex) + "条记录，正在准备导入...");
  var self = this;
  this._dao = new Dao();
  this._dao.bindService = false;
  this._dao.autoConnected = false;
  this._dao.isTransaction = false;
  this._importFields = {};
  this._pkFields = [];
  let count = 0;
  var titleCols = data[req.options.titleRowIndex - 1];
  this._dao.model = server.loadModule(req.options.model.path);
  var needCheckFields = [];
  for (var index = 0; index < req.options.fields.length; index++) {
    let field = req.options.fields[index];
    if (field.options || field.notNull) needCheckFields.push(index);
    field.colIndex = -1
    for (var i = 0; i < titleCols.length; i++) {
      if (titleCols[i] == field.column) {
        field.colIndex = i;
        break;
      }
    }

    if (field.colIndex == -1) {
      if (field.key) {
        self.onLogicError(7, '缺少主键列[' + field.column + ']，无法判断数据记录是否已存在！');
        return;
      }
      if (!field.value) continue;
    }
    for (let fieldName in self._dao.model.fields) {
      if (fieldName == field.field) {
        field.type = self._dao.model.fields[fieldName].type;
        break;
      }
    }
    self._importFields[field.field] = field;
    if (field.key) self._pkFields.push(field.field);
    count++;
  }
  if (count == 0) {
    this.onLogicError(8, '未解析到有效的表格标题，请确认表格格式是否与导入设置一致！');
    return;
  }
  //如果字段限定了可选项，或者不能为空，则需要判断Excel数据是否满足要求
  if (needCheckFields.length > 0) {
    for (var i = parseInt(req.options.titleRowIndex); i < data.length; i++) {
      for (var j = 0; j < needCheckFields.length; j++) {
        var field = req.options.fields[needCheckFields[j]];
        var cellData = data[i][field.colIndex];
        if (field.notNull && (!cellData || (cellData + "").trim() == "")) {
          this.onLogicError(9, field.column + '不能为空！');
          return;
        }
        if (field.options && cellData && !field.options.contains(cellData)) {
          this.onLogicError(9, field.column + '的值不能为' + cellData + '！');
          return;
        }
      }
    }
  }
  this._dao.open(function () {
    if(self._pkFields.length>0) {
      self._dao.query({fields: self._pkFields.join(",")}, function(keys) {
        self._keys = keys;
        self.exeImport(req, res, data, index, cb);
      });
    } else {
      self._keys = [];
      self.exeImport(req, res, data, index, cb);
    }
  });
};

ImportExcel.prototype.exeImport =  function(req, res, data, index, cb) {
  //开启后台运行
  var self = this;
  this.runInBackground(res);
  this._dao.onError = function(err) {
    self.onImportError(req, err.message, cb);
  };
  this.serviceInfo = server.tmpBGServices[this.id];
  this.serviceInfo.total = data.length - req.options.titleRowIndex;
  this.serviceInfo.finish = 0;
  this.importRow(req, res, data, parseInt(req.options.titleRowIndex), cb);
};

ImportExcel.prototype.importRow = function (req, res, data, index, cb) {
  if (index > req.options.titleRowIndex) this.serviceInfo.finish++;
  if (index >= data.length) {
    logger.log("导入完毕。");
    this._importFields = null;
    this._pkFields = null;
    this._keys = null;
    this._dao.close();
    server.tmpBGServices[this.id].status = 1;
    if (cb) {
      cb(data);
    } else {
      data = null;
      this.end(res);
    }
    return;
  }

  let dataRow = data[index];

  //生成当前行判断是否记录已存在的查询条件
  let pkWhere = "";
  for (var i = 0; i < this._pkFields.length; i++) {
    if (i > 0) pkWhere += " and ";
    let keyFieldName = this._pkFields[i];
    let keyField = this._importFields[keyFieldName];
    pkWhere += " " + keyFieldName + "=";
    let pkData = dataRow[keyField.colIndex];
    //如果主键字段为空，则不处理该行
    if (!pkData || pkData == "") {
      this.importRow(req, res, data, index + 1, cb);
      return;
    }
    if (keyField.type == "int" || keyField.type == "float"
      || keyField.type == 'double' || keyField.type == 'numeric') {
      pkWhere += pkData;
    } else {
      pkWhere += "'" + pkData + "'";
    }
  }

  let isExists = false;
  for (var i = 0; i < this._keys.length; i++) {
    let keyRow = this._keys[i];
    let isMatch = true;
    for (var j = 0; j < this._pkFields.length; j++) {
      let keyFieldName = this._pkFields[j];
      let keyField = this._importFields[keyFieldName];
      if (keyRow[keyFieldName] != dataRow[keyField.colIndex]) {
        isMatch = false;
        break;
      }
    }
    if (isMatch) {
      isExists = true;
      break;
    }
  }
  //如果记录不存在，则直接插入
  if (!isExists) {
    this.insertRow(req, res, data, dataRow, index, true, cb);
    return;
  }
  //如果是删除重建
  if (parseInt(req.options.importType) == 2) {
    var self = this;
    this._dao.delete({ where: pkWhere }, function () {
      self.insertRow(req, res, data, dataRow, index, false, cb);
    });
  } else {
    this.updateRow(req, res, data, dataRow, pkWhere, index, cb);
  }
};


ImportExcel.prototype.formatData = function (field, cellData) {
  if ((!cellData && cellData != 0) || (cellData + "").trim() == "") {
    return null;
  }
  if (field.type == "datetime" || field.type == 'date') {
    return this.formatDate(cellData);
  } else if (field.type == "int") {
    cellData = parseInt(cellData);
    if (isNaN(cellData)) return null;
  } else if (field.type == "float" || field.type == "double" || field.type == "numeric") {
    cellData = parseFloat(cellData);
    if (isNaN(cellData)) return null;
  } else {
    return cellData + "";
  }
};

ImportExcel.prototype.insertRow = function (req, res, data, dataRow, index, isNew, cb) {
  var newData = {};
  var newKey = {};
  for (var fieldName in this._importFields) {
    let field = this._importFields[fieldName];
    let cellData = field.value ? field.value : this.formatData(field, dataRow[field.colIndex]);
    //忽略空数据
    if ((!cellData && cellData != 0) || cellData == "") {
      continue;
    }
    newData[fieldName] = cellData;
    if (field.key) newKey[fieldName] = cellData;
  }
  var self = this;
  try {
    this._dao.create(newData, function (arg1, arg2, err) {
      if (err) {
        self.onImportError(req, err.message, cb);
        return;
      }
      //将新加的行存入主键集合
      if (isNew && self._pkFields.length > 0) self._keys.push(newKey);
      self.importRow(req, res, data, index + 1, cb);
    });
  } catch (err) {
    this.onImportError(req, err.message, cb);
  }
};

ImportExcel.prototype.updateRow = function (req, res, data, dataRow, where, index, cb) {
  var newData = {};
  var count = 0;
  for (var fieldName in this._importFields) {
    let field = this._importFields[fieldName];
    let cellData = field.value ? field.value : this.formatData(field, dataRow[field.colIndex]);
    //忽略主键和空数据
    if (field.key || (!cellData && cellData != 0) || cellData == "") {
      continue;
    }
    newData[fieldName] = cellData;
    count++;
  }
  //如果没有任何可以修改的数据，则跳过该行
  if (count == 0) {
    this.importRow(req, res, data, index + 1, cb);
    return;
  }
  let options = {
    where: where
  };
  var self = this;
  try {
    this._dao.update(newData, options, function (arg1, arg2, err) {
      if (err) {
        self.onImportError(req, err.message, cb);
        return;
      }
      self.importRow(req, res, data, index + 1, cb);
    });
  } catch (err) {
    this.onImportError(req, err.message, cb);
  }
};

ImportExcel.prototype.formatDate = function (data) {
  if (typeof (data) == "string") {
    if (data.indexOf(":") > 0) {
      return new Date(data);
    } else if (data.indexOf("-") > 0 || data.indexOf("/") > 0) {
      return new Date(data + " 0:00:00");
    } else {
      return null;
    }
  } else if (typeof data === 'number' && !isNaN(data)) {
    var date_num = parseInt(data);
    var time_num = data - date_num;
    var date = new Date(1900, 0, 0, 0, 0, 0);
    date.setDate(date_num - 1);
    date.setMonth(date.getMonth() + 1);
    date.setSeconds(time_num * 60 * 60 * 12 * 2);
    date.setSeconds(0);
    return date;
  } else {
    return null;
  }
};

ImportExcel.prototype.onImportError = function (req, err, cb) {
  this._importFields = null;
  this._pkFields = null;
  this._keys = null;
  //if(this._dao.isConnected) this._dao.close();
  var res = { code: 500, message: err };
  var tmpBGService = server.tmpBGServices[this.id];
  tmpBGService.status = -1;
  tmpBGService.endTime = new Date();
  tmpBGService.error = res;
  if (cb) {
    cb(req, res);
  } else {
    this.end(res);
  }
};
