var Service = require('../../../../common/Service');
var utils = require('../../../../common/utils');
var Org = require('../../dao/org');
var User = require('../../dao/user');
var Dep = require('../../dao/dep');
var Dep_User = require('../../dao/dep_user');
var Role_User = require('../../dao/role_user');
var Role = require('../../dao/role');
var util = require('util');

/**
 * 用户注册
 */
var Regist = function () {
  Service.call(this);
  this.checkLogin = false;
  this.checkAuthority = false;
};

util.inherits(Regist, Service);

module.exports = Regist;

/**
 * 处理服务请求
 * @param {Object} req 服务请求对象
 * @param {Object} res 服务响应对象
 */
Regist.prototype.process = function (req, res, cb) {
  if (!req.mobile) {
    this.onLogicError(1, '请提供手机号码！');
    return;
  }
  if (!utils.isPhone(req.mobile)) {
    this.onLogicError(2, '请提供有效的手机号码！');
    return;
  }
  if (!req.orgCode) {
    this.onLogicError(3, '请提供组织代码！');
    return;
  }
  if (utils.haveEspecialChar(req.orgCode)) {
    this.onLogicError(4, '请提供有效的组织代码！');
    return;
  }
  if (!req.name) {
    this.onLogicError(7, '请提供姓名！');
    return;
  }
  if (req.name.length > 10) {
    this.onLogicError(7, '姓名长度超出限制！');
    return;
  }
  if (utils.haveEspecialChar(req.name)) {
    this.onLogicError(7, '姓名中不能包含特殊字符！');
    return;
  }
  if (!req.password) {
    this.onLogicError(8, '请提供密码！');
    return;
  }
  if (req.password.length > 20) {
    this.onLogicError(8, '密码长度超出限制！');
    return;
  }
  if (utils.haveEspecialChar(req.password)) {
    this.onLogicError(8, '密码不能包含特殊字符！');
    return;
  }
  var sex = 0;
  if (req.sex == 1) sex = 1;

  /*if(!req.verifyCode) {
    this.onLogicError(4, '请提供验证码！');
    return;
  }
  var codeInfo = app.cache.registVerifyCodes[req.mobile];
  if(!codeInfo || codeInfo.code!=req.verifyCode) {
    this.onLogicError(5, '验证码无效！');
    return;
  }
  if(Math.abs((codeInfo.time - (new Date())))/(1000*60)>1) {
    this.onLogicError(6, '验证码已过期，请重新获取！');
    return;
  }*/

  var self = this;
  var daoOrg = new Org(this);
  daoOrg.query({
    fields: 'id,allow_regist,regist_url',
    where: 'code="' + req.orgCode + '"'
  }, function (orgs) {
    if (orgs.length == 0) {
      self.onLogicError(4, '组织不存在！');
      return;
    }
    var org = orgs[0];
    if (org.allow_regist != 1) {
      self.onLogicError(4, '该组织目前不允许注册！');
      return;
    }
    if (org.regist_url && !req.isExeRegist) {
      res.regist_url = org.regist_url;
      self.end(res);
      return;
    }
    var dao = new User(self);
    dao.query({
        fields: 'id',
        where: 'org_id=' + org.id + ' and (mobile="' + req.mobile + '" or loginname="' + req.mobile + '")'
      },
      function (rows) {
        if (rows.length > 0) {
          self.onLogicError(10, '该手机号已注册，不能重复注册！');
          return;
        }

        var user = {};
        user.org_id = org.id;
        user.code = req.mobile;
        user.name = req.name.replace(/ /g, '');
        user.loginname = req.mobile;
        user.password = req.password.replace(/ /g, '');
        user.mobile = req.mobile;
        user.sex = sex;
        user.status = 0;
        user.creater_id = 0;
        user.createtime = new Date();
        user.create_group = 1;
        user.look_all = 0;
        user.look_self_bureau = 0;
        user.look_self_dep = 0;
        user.look_leader = 0;
        user.look_under = 0;
        if (req.post) user.post = req.post;
        if (req.province) user.province = req.province;
        if (req.city) user.city = req.city;
        if (req.district) user.district = req.district;
        if (req.address) user.address = req.address;
        if (req.nickname) user.nickname = req.nickname;
        dao.create(user, function (ret) {
          user.id = ret.insertId;
          user.imStatus = 'offline';
          user.head_id = 0;
          user.deps = [];
          user.roles = [];
          user.groups = [];
          user.messages = [];
          delete user.password;
          delete user.status;
          delete user.creater_id;
          delete user.createtime;
          self.addDepUser(req, res, org, user, cb);
        });
      });
  });
};

//将用户添加到注册用户部门内
Regist.prototype.addDepUser = function (req, res, org, user, cb) {
  var self = this;
  var daoDep = new Dep(this);
  daoDep.query({
    fields: 'id',
    where: 'org_id=' + org.id +
      ' and name="注册用户"'
  }, function (deps) {
    if (deps.length == 0 && req.depId) {
      if (cb) {
        cb(user);
      } else {
        self.end(res);
      }
      return;
    }
    var dep = deps[0];
    if (!req.depId) dep.id = req.depId; //指定部门
    var dao = new Dep_User(self);
    dao.create({
      org_id: user.org_id,
      dep_id: dep.id,
      user_id: user.id,
      creater_id: user.id,
      createtime: new Date()
    }, function () {
      if (cb) {
        cb(user);
        return;
      }

      //self.addRoleUser(req, res, org, user);

      var args = {};
      args.user = user;
      args.arrDepId = [dep.id];

      self.requestIMService('sys/service/cache/addUser',
        args, req,
        function (resIM) {
          if (resIM.code != 0) {
            self.onError(resIM);
            return;
          }
          if (cb) {
            cb(user);
          } else {
            self.end(res);
          }
        });
    });
  });
};

//将用户添加到注册用户角色内
Regist.prototype.addRoleUser = function (req, res, org, user, cb) {
  var self = this;
  var daoRole = new Role(this);
  daoRole.query({
    fields: 'id',
    where: 'org_id=' + org.id + ' and name="注册用户"'
  }, function (rows) {
    if (rows.length == 0) {
      if (cb) {
        cb(user);
        return;
      }
      self.emit('end', res);
      return;
    }
    var dao = new Role_User(self);
    dao.create({
      org_id: org.id,
      role_id: rows[0].id,
      user_id: user.id,
      creater_id: user.id,
      createtime: new Date()
    }, function () {
      if (cb) {
        cb(user);
        return;
      }
      self.emit('end', res);
    });
  });
};