var taskMsgGrids = {};

/**
 * 数据表格
 */
function TaskMsgGrid() {
  this.id = null; //编号
  this.gridDiv = null; //数据表对应的div
  this.getDataService = this.getListService = null; //获得数据列表的Service名称
  this.deleteService = null; //删除数据的Service名称
  this.dataMember = null; //获得数据列表Service返回的存储列表数据的对象名称
  this.dataKey = null; //数据的主键名称，多个主键用英文逗号分隔
  this.templateRow = null; //模板行
  this.dataList = null; //数据列表
  this.dataPage = null; //数据页
  this.allowPaging = false; //是否允许翻页
  this.toolBarDiv = null; //工具栏div
  this.headerDiv = null; //表头div
  this.footerDiv = null; //表尾div
  this.dataDiv = null; //表体div
  this.dataRowDiv = null; //数据行div
  this.showHeader = true; //是否显示表头
  this.showSumer = false; //是否显示合计行
  this.showFooter = true; //是否显示表尾
  this.showCheckbox = false; //是否显示选择框
  this.showBackButton = false; //是否显示返回按钮
  this.pageSize = this.defaultPageSize = 20; //默认每页最多显示的记录数
  this.onAddedDataRow = null; //当添加完数据行后需要执行的回调函数
  this.onBack = null; //当点击返回按钮后需要执行的回调函数
  this.columns = []; //列集合

  var thisGrid = this;

  //初始化
  this.init = function (gridDiv) {
    this.gridDiv = gridDiv;
    this.dataDiv = this.gridDiv.children(".TaskMsgGridDataDiv");
    if (this.dataDiv.length == 0) return;
    this.dataRowDiv = this.dataDiv.children(".TaskMsgGridDataRow");
    if (this.dataRowDiv.length == 0) return;
    //获得表格相关属性
    if (this.gridDiv.attr("getListService")) this.getListService = this.gridDiv.attr("getListService");
    if (this.gridDiv.attr("deleteService")) this.deleteService = this.gridDiv.attr("deleteService");
    if (this.gridDiv.attr("dataMember")) this.dataMember = this.gridDiv.attr("dataMember");
    if (this.gridDiv.attr("dataKey")) this.dataKey = this.gridDiv.attr("dataKey");
    if (this.gridDiv.attr("allowPaging") == "true") this.allowPaging = true;
    if (this.gridDiv.attr("showHeader") == "false") this.showHeader = false;
    if (this.gridDiv.attr("showSumer") == "true") this.showSumer = true;
    if (this.gridDiv.attr("showFooter") == "false" && !this.allowPaging) this.showFooter = false;
    if (this.gridDiv.attr("showCheckbox") == "true") this.showCheckbox = true;
    if (this.gridDiv.attr("showBackButton") == "true") this.showBackButton = true;
    if (this.gridDiv.attr("onAddedDataRow")) this.onAddedDataRow = this.gridDiv.attr("onAddedDataRow");
    if (this.gridDiv.attr("onBack")) this.onBack = this.gridDiv.attr("onBack");

    //设置工具栏
    if (this.gridDiv.children(".TaskMsgGridToolBar").length == 0) {
      this.gridDiv.append("<div class=\"TaskMsgGridToolBar\"></div>");
    }
    this.toolBarDiv = this.gridDiv.children(".TaskMsgGridToolBar");
    //if(this.getListService && this.dataMember) {
    this.toolBarDiv.append("<img src=\"" + TaskMsgClientAPIDir + "images/refresh24.png\" class=\"TaskMsgGridRefreshBtn\" title=\"刷新\" />");
    this.toolBarDiv.children(".TaskMsgGridRefreshBtn").click(function () {
      thisGrid.loadData();
    });
    //}
    if (this.showBackButton) {
      this.toolBarDiv.append("<img src=\"" + TaskMsgClientAPIDir + "images/back24.png\" class=\"TaskMsgGridBackBtn\" title=\"后退\" />");
      this.toolBarDiv.children(".TaskMsgGridBackBtn").click(function () {
        if (thisGrid.onBack) {
          eval(thisGrid.onBack);
        } else {
          history.back();
        }
      });
    }

    if (this.gridDiv.attr("showSearchInput") == "true") {
      this.toolBarDiv.append("<div style=\"float: right;width:200px; " +
        "height:24px; border:1px solid #CCCCCC; margin-top:17px; margin-right: 10px;\">" +
        "<input id=\"input_"+this.id+"\" type=\"text\" style=\"float: left;width:176px;height:22px; border:0px;\" " +
        "onkeydown=\"if(event.keyCode==13 && this.value!=''){gridSearchButtonOnClick(this, '" + this.id + "')}\">" +
        "<img src=\"" + TaskMsgClientAPIDir + "images/find16.png\" style=\"float:left; margin-top:4px; " +
        "margin-left:4px; cursor:pointer\"  title=\"搜索\" " +
        "onclick=\"gridSearchButtonOnClick(this, '" + this.id + "')\"/></div>");
    }

    //设置表头、表体和表格最小宽度和模板列的宽度
    this.setHeader();
    //克隆模板行
    this.templateRow = this.dataRowDiv.clone();

    if ($(".TaskMsgGridMultiDelBtn").length > 0) {
      $(".TaskMsgGridMultiDelBtn").click(function () {
        if (!thisGrid.dataKey) {
          alert("请为TaskMsgGrid设置dataKey属性！");
          return;
        }
        if (!thisGrid.deleteService) {
          alert("请为TaskMsgGrid设置deleteService属性！");
          return;
        }
        if (!confirm("确定删除吗？")) return;
        var checkedValue = thisGrid.getCheckedValue();
        if (!checkedValue) {
          alert("请选择要删除的行！");
          return;
        }
        var args = {};
        if (thisGrid.dataKey.indexOf(",") > 0) {
          args[thisGrid.dataKey.replace(",", "_")] = checkedValue;
        } else {
          args[thisGrid.dataKey] = checkedValue;
        }
        (new TaskMsgService(thisGrid.deleteService)).request(args, thisGrid.onDeleteMultiResponse);
      });
    }

    this.loadData();
  };

  this.setHeader = function () {
    var minWidth = 42;
    if (this.gridDiv.children(".TaskMsgGridHeader").length > 0) {
      this.headerDiv = this.gridDiv.children(".TaskMsgGridHeader");
      if (this.showCheckbox) {
        this.headerDiv.prepend("<div style=\"width:20px;\">" +
          "<input type=\"checkbox\" class=\"TaskMsgGridCheckAll\" /></div>");
        this.headerDiv.find(".TaskMsgGridCheckAll").click(function () {
          var cbAll = this;
          thisGrid.dataDiv.find(".TaskMsgGridCheckbox").each(function () {
            $(this).get(0).checked = $(cbAll).is(":checked");
          });
        });
        this.dataRowDiv.prepend("<div style=\"text-align:center\">" +
          "<input type=\"checkbox\" class=\"TaskMsgGridCheckbox\" /></div>");
      }
      this.headerDiv.children("div").each(function () {
        minWidth += (getPixel($(this).css("width")) + 10);
        $(this).addClass("TaskMsgGridColumn");
        var cell = thisGrid.dataRowDiv.find("div:eq(" + $(this).index() + ")");
        cell.css("width", $(this).css("width"));
        cell.addClass("TaskMsgGridData");
        var column = {};
        column.width = $(this).css("width");
        column.align = $(this).css("text-align");
        column.allowSum = false;
        if ($(this).attr("allowSum") == "true") {
          column.allowSum = true;
          column.sumValue = 0;
        }
        thisGrid.columns.push(column);
      });
      if (!this.showHeader) {
        this.headerDiv.hide();
        this.dataDiv.css("top", "61px");
        this.dataDiv.css("border-top", "solid 1px #F0F0F0");
      }
    } else { //如果没有表头
      this.dataDiv.css("top", "61px");
      this.dataDiv.css("border-top", "solid 1px #F0F0F0");
      if (this.showCheckbox) {
        this.dataRowDiv.prepend("<div style=\"text-align:center\">" +
          "<input type=\"checkbox\" class=\"TaskMsgGridCheckbox\" /></div>");
      }
      this.dataRowDiv.children("div").each(function () {
        minWidth += (getPixel($(this).css("width")) + 10);
        $(this).addClass("TaskMsgGridData");
      });
    }
    this.gridDiv.css("minWidth", minWidth + "px");
  };

  /**
   * 加载数据
   * @return {[type]} [description]
   */
  this.loadData = function (p) {
    if (this.allowPaging) {
      if (p) {
        this.getPage(p);
      } else {
        var toPage = 1;
        if (getUrlArg("toPage")) toPage = parseInt(getUrlArg("toPage"));
        this.getPage(toPage);
      }
    } else this.getList();
  };

  /**
   * 获得选中的值
   * @return {[type]} [description]
   */
  this.getCheckedValue = function () {
    var ret = "";
    this.dataDiv.find(".TaskMsgGridCheckbox").each(function () {
      if ($(this).get(0).checked) {
        var keyValue = $(this).parent().parent().attr("key");
        if (ret != "") ret += ",";
        if (thisGrid.dataKey.indexOf(",") > 0) {
          ret += keyValue.replace(",", "_");
        } else {
          ret += keyValue;
        }
      }
    });
    return ret;
  };

  //请求数据
  this.getList = function (args) {
    for (var i = 0; i < this.columns.length; i++) {
      if (this.columns[i].allowSum) this.columns[i].sumValue = 0;
    }
    this.dataDiv.empty();
    this.dataDiv.append(
      "<div style=\"height:60px; width:100%; display:\">" +
      "  <img src=\"/images/loading.gif\" style=\"margin-top:14px; float:left; margin-left:20px;\" />" +
      "  <div style=\"float:left; line-height:60px; margin-left:20px;\">正在加载，请稍候...</div>" +
      "</div>");
    if (!this.getListService) return;
    (new TaskMsgService(this.getListService)).request(args, this.onGetListResponse);
  };

  //请求数据
  this.getPage = function (toPage, keyword) {
    var args = {};
    args.toPage = toPage;
    if (keyword) args.keyword = keyword;
    if (!this.footerDiv) args.pageSize = this.defaultPageSize;
    else args.pageSize = this.footerDiv.find(".TaskMsgGridPageSize").val();
    this.getList(args);
  };

  //添加行
  this.addRow = function (objData, isNew, updateRow, rowIndex) {
    if (thisGrid.allowPaging && isNew) {
      thisGrid.getPage(thisGrid.dataPage.toPage);
      return;
    }
    var divData = this.templateRow.clone();
    //设置行键
    if (this.dataKey) {
      divData.attr("key", objData[this.dataKey]);
      if (isNew) thisGrid.dataList[this.dataKey] = objData;
    }
    var data = replaceDataField(objData, divData.html(), rowIndex);
    data = exeExpress(data);
    divData.html(data);

    //设置删除链接的点击事件
    divData.find(".TaskMsgGridDelLink").click(function () {
      if (!thisGrid.dataKey) {
        alert("请为gridDiv设置dataKey属性！");
        return;
      }
      if (!thisGrid.deleteService) {
        alert("请为gridDiv设置deleteService属性！");
        return;
      }
      if (!confirm("确定删除吗？")) return;
      var args = {};
      var keyValue = $(this).closest(".TaskMsgGridDataRow").attr("key");
      if (thisGrid.dataKey.indexOf(",") > 0) {
        var keys = thisGrid.dataKey.split(",");
        var values = keyValue.split(",");
        for (var i = 0; i < keys; i++) {
          if (values.length > i) args[keys[i]] = values[i];
        }
      } else {
        args[thisGrid.dataKey] = keyValue;
      }
      (new TaskMsgService(thisGrid.deleteService)).request(args, thisGrid.onDeleteResponse);
    });
    if (thisGrid.dataDiv.children().length % 2 == 1) $(divData).addClass("TaskMsgGridDataRowEven");
    if (updateRow) {
      divData.insertAfter(updateRow);
    } else {
      thisGrid.dataDiv.append(divData);
    }
    if (thisGrid.onAddedDataRow) eval(thisGrid.onAddedDataRow + "(thisGrid, divData, objData)");
    divData.show();
    if (thisGrid.showSumer) {
      for (var i = 0; i < thisGrid.columns.length; i++) {
        if (thisGrid.columns[i].allowSum) {
          thisGrid.columns[i].sumValue += parseFloat(divData.children("div").get(i).innerHTML);
        }
      }
    }
    if (isNew) thisGrid.setFooter();
  };

  //修改行
  this.updateRow = function (args) {
    if (!this.dataKey) return;
    var keyValue = args[this.dataKey];
    if (!keyValue) return;
    this.dataDiv.find(".TaskMsgGridDataRow").each(function () {
      if ($(this).attr("key") == (keyValue + "")) {
        thisGrid.addRow(args, false, $(this));
        $(this).remove();
        return false;
      }
    });
  };

  //处理获得数据列表的响应
  this.onGetListResponse = function (args, res) {
    $(".TaskMsgGridCheckAll").attr("checked",false);
    //隐藏加载动画
    thisGrid.dataDiv.empty();
    var dataList = res[thisGrid.dataMember];
    if(!dataList) return;
    if (thisGrid.allowPaging) {
      thisGrid.dataPage = dataList;
      thisGrid.dataList = thisGrid.dataPage.rows;
    } else {
      thisGrid.dataList = dataList;
    }
    if (!thisGrid.dataList) return;
    if (thisGrid.dataList instanceof Array) {
      for (var i = 0; i < thisGrid.dataList.length; i++) {
        thisGrid.addRow(thisGrid.dataList[i],null,null,i);
      }
    } else {
      var rowIndex = 0;
      for (var objId in thisGrid.dataList) {
        thisGrid.addRow(thisGrid.dataList[objId],null,null,rowIndex);
        rowIndex++;
      }
    }

    if (thisGrid.showSumer) thisGrid.setSumer();
    thisGrid.setFooter();
  };

  //设置合计行
  this.setSumer = function () {
    this.dataDiv.css("border-bottom", "solid 1px #F0F0F0");
    var divData = this.templateRow.clone();
    divData.children("div").each(function (index) {
      if (thisGrid.columns[index].allowSum) {
        $(this).html(thisGrid.columns[index].sumValue);
      } else {
        $(this).html("");
      }
    });
    divData.find("div:first").html("合计");
    var sumerHtml = "<div class=\"TaskMsgGridSumer\"";
    if (!this.showFooter) sumerHtml += " style=\"bottom:0\"";
    sumerHtml += ">" + divData.html() + "</div>";
    this.gridDiv.append(sumerHtml);
  };

  //设置表尾
  this.setFooter = function () {
    if (!thisGrid.showFooter) {
      if (thisGrid.gridDiv.children(".TaskMsgGridFooter").length > 0) thisGrid.gridDiv.children(".TaskMsgGridFooter").hide();
      if (thisGrid.showSumer) {
        thisGrid.dataDiv.css("bottom", "50px");
      } else {
        thisGrid.dataDiv.css("bottom", "10px");
      }
      thisGrid.dataDiv.css("border-bottom", "solid 1px #F0F0F0");
      return;
    }
    if (thisGrid.gridDiv.children(".TaskMsgGridFooter").length == 0) {
      thisGrid.gridDiv.append("<div class=\"TaskMsgGridFooter\"></div>");
    }
    thisGrid.footerDiv = thisGrid.gridDiv.children(".TaskMsgGridFooter");
    if (thisGrid.allowPaging) {
      var prevPage = thisGrid.dataPage.toPage - 1;
      if (prevPage < 1) prevPage = 1;
      var nextPage = thisGrid.dataPage.toPage + 1;
      if (nextPage > thisGrid.dataPage.pageCount) nextPage = thisGrid.dataPage.pageCount;
      var pageHtml = "<span class=\"TaskMsgGridPageBtn\" onclick=\"gridGotoPage(this,1)\">首页</span>&nbsp;&nbsp;" +
        "<span class=\"TaskMsgGridPageBtn\" onclick=\"gridGotoPage(this," + prevPage + ")\">上一页</span>&nbsp;&nbsp;" +
        "<span class=\"TaskMsgGridPageBtn\" onclick=\"gridGotoPage(this," + nextPage + ")\">下一页</span>&nbsp;&nbsp;" +
        "<span class=\"TaskMsgGridPageBtn\" onclick=\"gridGotoPage(this," + thisGrid.dataPage.pageCount + ")\">尾页</span>" +
        "&nbsp;&nbsp;第 " + thisGrid.dataPage.toPage + " 页/共 " + thisGrid.dataPage.pageCount + " 页，" +
        thisGrid.dataPage.rowsCount + " 条记录&nbsp;&nbsp;<select class=\"TaskMsgGridPageSize\" onchange=\"gridChangePageSize(this)\">";
      pageHtml += "<option value=\"20\"";
      if (thisGrid.dataPage.pageSize == 20) pageHtml += " selected";
      pageHtml += ">20</option>";
      pageHtml += "<option value=\"30\"";
      if (thisGrid.dataPage.pageSize == 30) pageHtml += " selected";
      pageHtml += ">30</option>";
      pageHtml += "<option value=\"50\"";
      if (thisGrid.dataPage.pageSize == 50) pageHtml += " selected";
      pageHtml += ">50</option>";
      pageHtml += "<option value=\"100\"";
      if (thisGrid.dataPage.pageSize == 100) pageHtml += " selected";
      pageHtml += ">100</option></select>";
      thisGrid.footerDiv.html(pageHtml);
    } else {
      thisGrid.footerDiv.html("共 " + thisGrid.dataDiv.children().length + " 条记录");
    }
  };

  //响应删除操作
  this.onDeleteResponse = function (args, res) {
    if (thisGrid.allowPaging) {
      thisGrid.getPage(thisGrid.dataPage.toPage);
      return;
    }
    if (args.srcElement) {
      var curRow = $(args.srcElement).closest(".TaskMsgGridDataRow");
      var nextRow = curRow.next();
      while (nextRow.get(0)) {
        if (nextRow.hasClass("TaskMsgGridDataRowEven")) {
          nextRow.removeClass("TaskMsgGridDataRowEven");
        } else {
          nextRow.addClass("TaskMsgGridDataRowEven");
        }
        nextRow = nextRow.next();
      }
      curRow.remove();
    }
    if (thisGrid.dataKey) {
      delete thisGrid.dataList[args[thisGrid.dataKey]];
    }
    thisGrid.setFooter();
  };

  //响应删除多行的操作
  this.onDeleteMultiResponse = function (args, res) {
    if (thisGrid.allowPaging) {
      thisGrid.getPage(thisGrid.dataPage.toPage);
      return;
    }
    thisGrid.dataDiv.find(".TaskMsgGridCheckbox").each(function () {
      if ($(this).get(0).checked) {
        var keyValue = $(this).parent().parent().attr("key");
        if (keyValue) delete thisGrid.dataList[keyValue];
        $(this).parent().parent().remove();
      }
    });

    for (var i = 0; i < thisGrid.dataDiv.children().length; i++) {
      var row = thisGrid.dataDiv.children().get(i);
      if (i % 2 == 0) {
        if ($(row).hasClass("TaskMsgGridDataRowEven")) {
          $(row).removeClass("TaskMsgGridDataRowEven");
        }
      } else {
        if (!$(row).hasClass("TaskMsgGridDataRowEven")) {
          $(row).addClass("TaskMsgGridDataRowEven");
        }
      }
    }
    thisGrid.setFooter();
  };
}

function gridGotoPage(linkGoto, toPage) {
  var id = $(linkGoto).closest(".TaskMsgGrid").attr("id");
  var keyword  =$("#input_"+id).val();
  var grid = taskMsgGrids[id];
  grid.getPage(toPage,keyword);
}

function gridChangePageSize(slt) {
  var id = $(slt).closest(".TaskMsgGrid").attr("id");
  var keyword  =$("#input_"+id).val();
  var grid = taskMsgGrids[id];
  grid.getPage(grid.dataPage.toPage,keyword);
}

function gridSearchButtonOnClick(searchBtn, gridId) {
  var keyword = $(searchBtn).parent().find("input").val();
  if (keyword == "") {
    //alert("请输入要查询的关键字！");
    //searchBtn.focus();
    //return;
    keyword = null;
  }
  var grid = taskMsgGrids[gridId];
  if (grid.allowPaging) {
    grid.getPage(1, keyword);
  } else {
    var args = {};
    args.keyword = keyword;
    grid.getList(args);
  }
}

$(function () {
  $(".TaskMsgGrid").each(function (index) {
    var grid = new TaskMsgGrid();
    if ($(this).attr("id")) {
      grid.id = $(this).attr("id");
    } else {
      grid.id = "TaskMsgGrid_" + index;
    }
    taskMsgGrids[grid.id] = grid;
    grid.init($(this));
  });
});